@@ -1,29 +1,73 @@
-#include <ogr_geometry.h>
-
 #define GEOS_USE_ONLY_R_API // avoid using non-thread-safe GEOSxx functions without _r extension.
 #include <geos_c.h>
 
 #include <Rcpp.h>
 
-#include "gdal.h"
+#include "wkb.h"
+
+static void __errorHandler(const char *fmt, ...) {
+
+    char buf[BUFSIZ], *p;
+    va_list(ap);
+    va_start(ap, fmt);
+    vsprintf(buf, fmt, ap);
+    va_end(ap);
+    p = buf + strlen(buf) - 1;
+    if(strlen(buf) > 0 && *p == '\n') *p = '\0';
+
+	Rcpp::Function error("stop");
+    error(buf);
+
+    return;
+}
+
+static void __warningHandler(const char *fmt, ...) {
+
+    char buf[BUFSIZ], *p;
+    va_list(ap);
+    va_start(ap, fmt);
+    vsprintf(buf, fmt, ap);
+    va_end(ap);
+    p = buf + strlen(buf) - 1;
+    if(strlen(buf) > 0 && *p == '\n') *p = '\0';
+
+	Rcpp::Function warning("warning");
+	warning(buf);
+    
+    return;
+}
+
+GEOSContextHandle_t CPL_geos_init(void) {
+    return(initGEOS_r((GEOSMessageHandler) __warningHandler, (GEOSMessageHandler) __errorHandler));
+}
+
+void CPL_geos_finish(GEOSContextHandle_t ctxt) {
+	finishGEOS_r(ctxt);
+}
 
 std::vector<GEOSGeom> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::List sfc) {
-	std::vector<OGRGeometry *> ogr = ogr_from_sfc(sfc, NULL);
-	std::vector<GEOSGeom> g(ogr.size());
-	for (int i = 0; i < ogr.size(); i++) {
-		g[i] = ogr[i]->exportToGEOS(hGEOSCtxt);
-		delete ogr[i];
+	double precision = sfc.attr("precision");
+	Rcpp::List wkblst = CPL_write_wkb(sfc, false, native_endian(), "XY", precision);
+	std::vector<GEOSGeom> g(sfc.size());
+	for (int i = 0; i < sfc.size(); i++) {
+		Rcpp::RawVector r = wkblst[i];
+		g[i] = GEOSGeomFromWKB_buf_r(hGEOSCtxt, &(r[0]), r.size());
 	}
 	return g;
 }
 
 Rcpp::List sfc_from_geometry(GEOSContextHandle_t hGEOSCtxt, std::vector<GEOSGeom> geom) {
-	std::vector<OGRGeometry *> ogr(geom.size());
+	Rcpp::List out(geom.size());
 	for (int i = 0; i < geom.size(); i++) {
-		ogr[i] = OGRGeometryFactory::createFromGEOS(hGEOSCtxt, geom[i]);
+		size_t size;
+		unsigned char *buf = GEOSGeomToWKB_buf_r(hGEOSCtxt, geom[i], &size);
+		Rcpp::RawVector raw(size);
+		memcpy(&(raw[0]), buf, size);
+		free(buf);
+		out[i] = raw;
 		GEOSGeom_destroy_r(hGEOSCtxt, geom[i]);
 	}
-	return sfc_from_ogr(ogr, true); // destroy
+	return CPL_read_wkb(out, false, native_endian());
 }
 
 Rcpp::NumericVector get_dim(double dim0, double dim1) {
@@ -48,14 +92,14 @@ Rcpp::IntegerVector get_which(Rcpp::LogicalVector row) {
 bool chk_(char value) {
 	if (value == 2)
 		throw std::range_error("GEOS exception");
-	return value == 1;
+	return value; // 1: true, 0: false
 }
 
 // [[Rcpp::export]]
 Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, double par = 0.0, 
 		bool sparse = true) {
 
-	GEOSContextHandle_t hGEOSCtxt = OGRGeometry::createGEOSContext();
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 
 	std::vector<GEOSGeom> gmv0 = geometries_from_sfc(hGEOSCtxt, sfc0);
 	std::vector<GEOSGeom> gmv1 = geometries_from_sfc(hGEOSCtxt, sfc1);
@@ -147,38 +191,39 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 		GEOSGeom_destroy_r(hGEOSCtxt, gmv0[i]);
 	for (int i = 0; i < gmv1.size(); i++)
 		GEOSGeom_destroy_r(hGEOSCtxt, gmv1[i]);
-	OGRGeometry::freeGEOSContext(hGEOSCtxt);
+	CPL_geos_finish(hGEOSCtxt);
 	return ret_list;
 }
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_geos_is_valid(Rcpp::List sfc) { 
-	GEOSContextHandle_t hGEOSCtxt = OGRGeometry::createGEOSContext();
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 	std::vector<GEOSGeom> gmv = geometries_from_sfc(hGEOSCtxt, sfc);
 	Rcpp::LogicalVector out(gmv.size());
 	for (int i = 0; i < out.length(); i++) {
 		out[i] = chk_(GEOSisValid_r(hGEOSCtxt, gmv[i]));
 		GEOSGeom_destroy_r(hGEOSCtxt, gmv[i]);
 	}
-	OGRGeometry::freeGEOSContext(hGEOSCtxt);
+	CPL_geos_finish(hGEOSCtxt);
 	return out;
 }
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_geos_is_simple(Rcpp::List sfc) { 
-	GEOSContextHandle_t hGEOSCtxt = OGRGeometry::createGEOSContext();
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 	Rcpp::LogicalVector out(sfc.length());
 	std::vector<GEOSGeom> g = geometries_from_sfc(hGEOSCtxt, sfc);
 	for (size_t i = 0; i < g.size(); i++) {
 		out[i] = chk_(GEOSisSimple_r(hGEOSCtxt, g[i]));
 		GEOSGeom_destroy_r(hGEOSCtxt, g[i]);
 	}
+	CPL_geos_finish(hGEOSCtxt);
 	return out;
 }
 
 // [[Rcpp::export]]
 Rcpp::List CPL_geos_union(Rcpp::List sfc) { 
-	GEOSContextHandle_t hGEOSCtxt = OGRGeometry::createGEOSContext();
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 	std::vector<GEOSGeom> gmv = geometries_from_sfc(hGEOSCtxt, sfc);
 	GEOSGeom gc = GEOSGeom_createCollection_r(hGEOSCtxt, GEOS_GEOMETRYCOLLECTION, gmv.data(), gmv.size());
 	std::vector<GEOSGeom> gmv_out(1);
@@ -189,7 +234,7 @@ Rcpp::List CPL_geos_union(Rcpp::List sfc) {
 #endif
 	GEOSGeom_destroy_r(hGEOSCtxt, gc);
 	Rcpp::List out(sfc_from_geometry(hGEOSCtxt, gmv_out)); // destroys gmv_out
-	OGRGeometry::freeGEOSContext(hGEOSCtxt);
+	CPL_geos_finish(hGEOSCtxt);
 	return out;
 }
 
@@ -206,51 +251,38 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 		double dTolerance = 0.0, bool preserveTopology = false, 
 		int bOnlyEdges = 1, double dfMaxLength = 0.0) {
 
-	GEOSContextHandle_t hGEOSCtxt = OGRGeometry::createGEOSContext();
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init(); 
+
 	std::vector<GEOSGeom> g = geometries_from_sfc(hGEOSCtxt, sfc);
 	std::vector<GEOSGeom> out(sfc.length());
 
-	// std::vector<OGRGeometry *> g = ogr_from_sfc(sfc, NULL);
-	// std::vector<OGRGeometry *> out(g.size());
-
 	if (op == "buffer") {
 		for (size_t i = 0; i < g.size(); i++)
-			// out[i] = g[i]->Buffer(bufferDist, nQuadSegs);
 			out[i] = chkNULL(GEOSBuffer_r(hGEOSCtxt, g[i], bufferDist, nQuadSegs));
 	} else if (op == "boundary") {
 		for (size_t i = 0; i < g.size(); i++)
-			// out[i] = g[i]->Boundary();
 			out[i] = chkNULL(GEOSBoundary_r(hGEOSCtxt, g[i]));
 	} else if (op == "convex_hull") {
 		for (size_t i = 0; i < g.size(); i++)
-			// out[i] = g[i]->ConvexHull();
 			out[i] = chkNULL(GEOSConvexHull_r(hGEOSCtxt, g[i]));
 	} else if (op == "union_cascaded") {
 		for (size_t i = 0; i < g.size(); i++)
-			// out[i] = g[i]->UnionCascaded();
 			out[i] = chkNULL(GEOSUnionCascaded_r(hGEOSCtxt, g[i]));
 	} else if (op == "simplify") {
 		for (size_t i = 0; i < g.size(); i++)
-			// out[i] = preserveTopology ?  g[i]->SimplifyPreserveTopology(dTolerance) : 
-			//		g[i]->Simplify(dTolerance);
 			out[i] = preserveTopology ? chkNULL(GEOSTopologyPreserveSimplify_r(hGEOSCtxt, g[i], dTolerance)) :
 					chkNULL(GEOSSimplify_r(hGEOSCtxt, g[i], dTolerance));
 	} else if (op == "polygonize") {
 		for (size_t i = 0; i < g.size(); i++)
-			// out[i] = g[i]->Polygonize();
 			out[i] = chkNULL(GEOSPolygonize_r(hGEOSCtxt, &(g[i]), 1)); // xxx
 	} else if (op == "centroid") {
 		for (size_t i = 0; i < g.size(); i++) {
-			// OGRPoint *gm = new OGRPoint;
-			// g[i]->Centroid(gm);
-			// out[i] = gm;
 			out[i] = chkNULL(GEOSGetCentroid_r(hGEOSCtxt, g[i]));
 		}
 	} else
 #if GEOS_VERSION_MAJOR >= 3 && GEOS_VERSION_MINOR >= 4
 	if (op == "triangulate") {
 		for (size_t i = 0; i < g.size(); i++)
-			// out[i] = g[i]->DelaunayTriangulation(dTolerance, bOnlyEdges);
 			out[i] = chkNULL(GEOSDelaunayTriangulation_r(hGEOSCtxt, g[i], dTolerance, bOnlyEdges));
 	} else
 #endif
@@ -260,48 +292,76 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 		GEOSGeom_destroy_r(hGEOSCtxt, g[i]);
 
 	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, out)); // destroys out
-	OGRGeometry::freeGEOSContext(hGEOSCtxt);
+	CPL_geos_finish(hGEOSCtxt);
 	ret.attr("crs") = sfc.attr("crs");
 	return ret;
 }
 
+GEOSGeometry *chkNULLcnt(GEOSContextHandle_t hGEOSCtxt, GEOSGeometry *value, size_t *n) {
+	if (value == NULL)
+		throw std::range_error("GEOS exception");
+	if (!chk_(GEOSisEmpty_r(hGEOSCtxt, value)))
+		*n = *n + 1;
+	return value;
+}
+
 // [[Rcpp::export]]
-Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfc, Rcpp::List sf0) {
+Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 
-	GEOSContextHandle_t hGEOSCtxt = OGRGeometry::createGEOSContext();
-	std::vector<GEOSGeom> g = geometries_from_sfc(hGEOSCtxt, sfc);
-	std::vector<GEOSGeom> g0 = geometries_from_sfc(hGEOSCtxt, sf0);
-	std::vector<GEOSGeom> out(sfc.length());
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	std::vector<GEOSGeom> x = geometries_from_sfc(hGEOSCtxt, sfcx);
+	std::vector<GEOSGeom> y = geometries_from_sfc(hGEOSCtxt, sfcy);
+	std::vector<GEOSGeom> out(x.size() * y.size());
 
+	size_t n = 0;
 	if (op == "intersection") {
-		for (size_t i = 0; i < g.size(); i++)
-			out[i] = chkNULL(GEOSIntersection_r(hGEOSCtxt, g[i], g0[0]));
+		for (size_t i = 0; i < y.size(); i++)
+			for (size_t j = 0; j < x.size(); j++)
+				out[i * x.size() + j] = chkNULLcnt(hGEOSCtxt, GEOSIntersection_r(hGEOSCtxt, x[j], y[i]), &n);
 	} else if (op == "union") {
-		for (size_t i = 0; i < g.size(); i++)
-			out[i] = chkNULL(GEOSUnion_r(hGEOSCtxt, g[i], g0[0]));
+		for (size_t i = 0; i < y.size(); i++)
+			for (size_t j = 0; j < x.size(); j++)
+				out[i * x.size() + j] = chkNULLcnt(hGEOSCtxt, GEOSUnion_r(hGEOSCtxt, x[j], y[i]), &n);
 	} else if (op == "difference") {
-		for (size_t i = 0; i < g.size(); i++)
-			out[i] = chkNULL(GEOSDifference_r(hGEOSCtxt, g[i], g0[0]));
+		for (size_t i = 0; i < y.size(); i++)
+			for (size_t j = 0; j < x.size(); j++)
+				out[i * x.size() + j] = chkNULLcnt(hGEOSCtxt, GEOSDifference_r(hGEOSCtxt, x[j], y[i]), &n);
 	} else if (op == "sym_difference") {
-		for (size_t i = 0; i < g.size(); i++)
-			out[i] = chkNULL(GEOSSymDifference_r(hGEOSCtxt, g[i], g0[0]));
+		for (size_t i = 0; i < y.size(); i++)
+			for (size_t j = 0; j < x.size(); j++)
+				out[i * x.size() + j] = chkNULLcnt(hGEOSCtxt, GEOSSymDifference_r(hGEOSCtxt, x[j], y[i]), &n);
 	} else 
 		throw std::invalid_argument("invalid operation"); // would leak g, g0 and out
-	// clean up:
-	for (size_t i = 0; i < g.size(); i++)
-		GEOSGeom_destroy_r(hGEOSCtxt, g[i]);
-	for (size_t i = 0; i < g0.size(); i++)
-		GEOSGeom_destroy_r(hGEOSCtxt, g0[i]);
+	// clean up x and y:
+	for (size_t i = 0; i < x.size(); i++)
+		GEOSGeom_destroy_r(hGEOSCtxt, x[i]);
+	for (size_t i = 0; i < y.size(); i++)
+		GEOSGeom_destroy_r(hGEOSCtxt, y[i]);
+	// trim results back to non-empty geometries:
+	std::vector<GEOSGeom> out2(n);
+	Rcpp::NumericMatrix m(n, 2); // and a set of 1-based indices to x and y
+	size_t k = 0, l = 0;
+	for (size_t i = 0; i < y.size(); i++) {
+		for (size_t j = 0; j < x.size(); j++) {
+			l = i * x.size() + j;
+			if (!chk_(GEOSisEmpty_r(hGEOSCtxt, out[l]))) { // keep:
+				out2[k] = out[l];
+				m(k, 0) = j + 1;
+				m(k, 1) = i + 1;
+				k++;
+				if (k > n)
+					throw std::range_error("invalid k");
+			} else // discard:
+				GEOSGeom_destroy_r(hGEOSCtxt, out[l]);
+		}
+	}
+	if (k != n)
+		throw std::range_error("invalid k (2)");
 
-	/* old GDAL impl:
-	OGRGeometryFactory f;
-	for (size_t i = 0; i < out.size(); i++)
-		if (out[i] == NULL)
-			out[i] = f.createGeometry(wkbGeometryCollection);
-	*/
-	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, out)); // destroys out
-	OGRGeometry::freeGEOSContext(hGEOSCtxt);
-	ret.attr("crs") = sfc.attr("crs");
+	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, out2)); // destroys out2
+	CPL_geos_finish(hGEOSCtxt);
+	ret.attr("crs") = sfcx.attr("crs");
+	ret.attr("idx") = m;
 	return ret;
 }
 
@@ -321,50 +381,3 @@ Rcpp::CharacterVector CPL_geos_relate(Rcpp::List sfc0, Rcpp::List sfc1) {
 	Rcpp::CharacterVector out = CPL_geos_binop(sfc0, sfc1, "relate", 0.0, false)[0];
 	return out;	
 }
-
-static void __errorHandler(const char *fmt, ...) {
-
-    char buf[BUFSIZ], *p;
-    va_list(ap);
-    va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
-    va_end(ap);
-    p = buf + strlen(buf) - 1;
-    if(strlen(buf) > 0 && *p == '\n') *p = '\0';
-
-	Rcpp::Function error("error");
-    error(buf);
-
-    return;
-}
-
-static void __warningHandler(const char *fmt, ...) {
-
-    char buf[BUFSIZ], *p;
-    va_list(ap);
-    va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
-    va_end(ap);
-    p = buf + strlen(buf) - 1;
-    if(strlen(buf) > 0 && *p == '\n') *p = '\0';
-
-	Rcpp::Function warning("warning");
-	warning(buf);
-    
-    return;
-}
-
-GEOSContextHandle_t geos_ctxt_ptr;  // global variable -- can it do any harm?
-
-// [[Rcpp::export]]
-void CPL_geos_init() {
-	// geos_ctxt_ptr = GEOS_init_r();
-	geos_ctxt_ptr =
-    	initGEOS_r((GEOSMessageHandler) __warningHandler, (GEOSMessageHandler) __errorHandler);
-}
-
-// [[Rcpp::export]]
-void CPL_geos_finish() {
-	// GEOS_finish_r(geos_ctxt_ptr); // needs context handler, we don't have one
-	finishGEOS_r(geos_ctxt_ptr);
-}
