@@ -1,6 +1,6 @@
-library(sf)
+suppressPackageStartupMessages(library(sf))
 # nc = st_read(system.file("gpkg/nc.gpkg", package="sf"))
-nc = st_read(system.file("shape/nc.shp", package="sf"))
+nc = st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE)
 
 st_distance(nc[1:10,], nc[1:10,])
 
